.o-text--primary{color:#43b1b0}.o-text--primary,.o-text--info{font-weight:600}.o-text--label{margin-right:0.4em}.o-list-item{border-bottom:1px solid #e6e6e6;margin:0 -2em;padding:1.5em 2em;font-size:1.15em;line-height:1.15em}.o-list-item:last-child{border-bottom:0}.o-panel-header{display:block;width:100%;background:#43b1b0;border-radius:.4em;border-bottom-left-radius:0;border-bottom-right-radius:0}.o-panel-header--failure{background:#f37e77}.o-panel-body{padding:0.618em 2em;background:#fafafa;border:1px solid #e6e6e6;border-radius:.4em;border-top-left-radius:0;border-top-right-radius:0}.o-panel-header__title{color:#fff;padding:0.5em 1em;margin:0}.o-alert{padding:0.618em 2em;background:#fafafa;color:#999;border:1px solid #e6e6e6;border-radius:.4em}.o-alert--success{border-color:#189370;background:#189370;color:#fff}.o-button{margin:1em 0}.o-button--success,.o-button--success[disabled]{color:#fff;background:#189370;border:1px solid #58b59b;pointer-events:none}.o-button--failure,.o-button--failure[disabled]{color:#fff;background:#f37e77;border:1px solid #fcf2f2;pointer-events:none}.o-collapse{margin:2em 0}.o-collapse-header{display:block;width:100%;background:#43b1b0;cursor:pointer;border-radius:.4em}.o-collapse-header--is-expanded{border-bottom-left-radius:0;border-bottom-right-radius:0}.o-collapse-header--is-expanded .o-collapse-header__title::after{content:"⌄";float:right;font-size:2em;margin-top:-0.2em;margin-right:-0.3em}.o-collapse-header--is-collapsed .o-collapse-header__title::after{content:"›";float:right;font-size:2em;margin-top:-0.1em}.o-collapse-header[disabled]{pointer-events:none;background:#d9d9d9}.o-collapse-header[disabled] .o-collapse-header__title{color:#9a9a9a}.o-collapse-header__title{color:#fff;padding:0.5em 1em;margin:0}.o-collapse-body{border-radius:.4em;border-top-left-radius:0;border-top-right-radius:0;background:#fafafa;border:1px solid #e6e6e6;padding:0.618em 2em}.o-input-group__item{margin:2em 1em}.o-input-group__label{font-size:1.15em;margin:0.618em}.o-button-group .o-button{border-radius:0;margin-right:0.1em}.o-button-group .o-button:first-child{border-top-left-radius:.4em;border-bottom-left-radius:.4em}.o-button-group .o-button:last-child{margin-right:0;border-top-right-radius:.4em;border-bottom-right-radius:.4em}.c-git-importer{padding:0 2em}
